/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.3 Automatisches Lernen
 * Ganglinien
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.gllernen.gllernen;

import com.bitctrl.Constants;
import com.bitctrl.util.Interval;

import de.bsvrz.sys.funclib.bitctrl.daf.BetriebsmeldungDaten;
import de.bsvrz.sys.funclib.bitctrl.daf.LogTools;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.MessQuerschnittAllgemein;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Führt das Lernen für einen Messquerschnitt aus. Da das Lernen als Thread
 * ausgelegt ist und sich nicht gegenseitig beeinflussen kann, können mehrere
 * Messquerschnitte gleichzeitig bearbeitet werden.
 *
 * @author BitCtrl Systems GmbH, Schumann
 */
public class LernenMQ implements Runnable {

	/** Der Logger der Klasse. */
	private final Debug log = Debug.getLogger();

	/** Der Messquerschnitt für den gelernt wird. */
	private final MessQuerschnittAllgemein mq;

	/** Der gesamte Lernzeitraum, kann mehrere Tage umfassen. */
	private final Interval lernzeitraum;

	/** Der maximale Vergleichsabstand für das Verschmelzen von Ganglinien. */
	private final int maxVergleichsAbstand;

	/**
	 * Initialisierung den Lernprozess.
	 *
	 * @param mq
	 *            der Messquerschnitt für den die Ganglinien gelernt werden.
	 * @param lernzeitraum
	 *            der Zeitraum in dem gelernt werden soll.
	 * @param maxVergleichsAbstand
	 *            der maximale Vergleichsabstand für das Verschmelzen von
	 *            Ganglinien.
	 */
	public LernenMQ(final MessQuerschnittAllgemein mq,
			final Interval lernzeitraum, final int maxVergleichsAbstand) {
		this.mq = mq;
		this.lernzeitraum = lernzeitraum;
		this.maxVergleichsAbstand = maxVergleichsAbstand;
	}

	/**
	 * Lernt jeden Tag im Lernzeitraum einzeln. Dies ist notwendig, da eine
	 * Ganglinien immer genau einen vollen Tag beschreibt.
	 */
	@Override
	public synchronized void run() {
		for (long t = lernzeitraum.getStart(); t < lernzeitraum
				.getEnd(); t += Constants.MILLIS_PER_DAY) {
			final long start, ende;
			final Lernen lernen;

			start = t;
			ende = start + Constants.MILLIS_PER_DAY;

			lernen = new Lernen(mq, new Interval(start, ende),
					maxVergleichsAbstand);
			lernen.lerne();
		}

		Verwaltung.getInstanz().getSemaphore().release();
		LogTools.log(log, new BetriebsmeldungDaten(mq.getSystemObject()),
				GlLernenMsg.InfoLernenErfolgreich, mq);
	}

}
